// @HEADER
// ***********************************************************************
// 
//                    Teuchos: Common Tools Package
//                 Copyright (2004) Sandia Corporation
// 
// Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
// license for use of this work by or on behalf of the U.S. Government.
// 
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//  
// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
// Questions? Contact Michael A. Heroux (maherou@sandia.gov) 
// 
// ***********************************************************************
// @HEADER

#ifndef TEUCHOS_REFCOUNTPTR_SHAREDPTR_CONVERSIONS_H
#define TEUCHOS_REFCOUNTPTR_SHAREDPTR_CONVERSIONS_H

#include "Teuchos_RefCountPtrBoostSharedPtrConversionsDecl.hpp"
#include "Teuchos_RefCountPtr.hpp"

template<class T>
Teuchos::RefCountPtr<T>
Teuchos::rcp( const boost::shared_ptr<T> &sptr )
{
  if(sptr.get()) {
    const RefCountPtrDeleter<T>
      *rcpd = boost::get_deleter<RefCountPtrDeleter<T> >(sptr);
    if(rcpd)
      return rcpd->ptr();
    return rcp(sptr.get(),DeallocBoostSharedPtr<T>(sptr),true);
  }
  return null;
}

template<class T>
boost::shared_ptr<T>
Teuchos::shared_pointer( const RefCountPtr<T> &rcp )
{
  if(rcp.get()) {
    const DeallocBoostSharedPtr<T>
      *dbsp = get_optional_dealloc<DeallocBoostSharedPtr<T> >(rcp);
    if(dbsp)
      return dbsp->ptr();
    return boost::shared_ptr<T>(rcp.get(),RefCountPtrDeleter<T>(rcp));
  }
  return boost::shared_ptr<T>();
}

#endif	// TEUCHOS_REFCOUNTPTR_SHAREDPTR_CONVERSIONS_H
